using System;
using JointPhysicsHelper;
using MVR.FileManagementSecure;
using SimpleJSON;

namespace MvKlein
{
    // macgruber 判断加载完成
    public class JointPhysicsHelper : MVRScript
    {
        bool myWasLoading = true;
        bool myNeedInit = true;
        private JSONStorableFloat springStorable;
        private JSONStorableFloat damperStorable;
        private JSONStorableFloat maxVelocity;
        public static string saveExt = "mv";
        public static string defaultPath = "Saves\\PluginData\\MvKlein\\MMDPhysics";
        public override void Init()
        {
            myWasLoading = true;
            myNeedInit = true;

            springStorable = new JSONStorableFloat(StringResource.SpringPercent.ToString(),15,0,100);
            damperStorable = new JSONStorableFloat(StringResource.DamperPercent.ToString(),50,0,100);
            maxVelocity = new JSONStorableFloat(StringResource.MaxVelocity.ToString(),0.5f,0,100);

            CreateSlider(springStorable);
            CreateSlider(damperStorable);
            CreateSlider(maxVelocity);

            var setBtn = CreateButton(StringResource.Apply.ToString());
            setBtn.button.onClick.AddListener(SetPhysicsValue);

            RegisterFloat(springStorable);
            RegisterFloat(damperStorable);
            RegisterFloat(maxVelocity);
            RegisterAction(new JSONStorableAction("Set Physics", SetPhysicsValue));

            var savePreset = CreateButton(StringResource.SavePreset.ToString());
            savePreset.button.onClick.AddListener(() => {
                FileManagerSecure.CreateDirectory(defaultPath);
                SuperController.singleton.NormalizeMediaPath(defaultPath+"\\");
                SuperController.singleton.GetMediaPathDialog(SavePreset, saveExt);

                // Update the browser to be a Save browser
                uFileBrowser.FileBrowser browser = SuperController.singleton.mediaFileBrowserUI;
                browser.defaultPath =  defaultPath;
                browser.SetTextEntry(true);
                browser.fileEntryField.text = String.Format("{0}.{1}", ((int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds).ToString(), saveExt);
                browser.ActivateFileNameField();
            });

            var loadPreset = CreateButton(StringResource.LoadPreset.ToString());
            loadPreset.button.onClick.AddListener(()=>
            {
                SuperController.singleton.NormalizeMediaPath(defaultPath+"\\");
                SuperController.singleton.GetMediaPathDialog(LoadPreset, saveExt);
            });
            var introTxt = new JSONStorableString("intro",StringResource.PluginInfo.ToString());
            var introField = CreateTextField(introTxt, true);
            introField.height = 280f;
            // ...
        }
        private void LoadPreset(string path) {
            if (String.IsNullOrEmpty(path)) { return; }
            var presetJson = this.LoadJSON(path);
            // springStorable;damperStorable;maxVelocity;
            try {
                if (presetJson["springStorable"] != null) { springStorable.val = presetJson["springStorable"].AsFloat;}
                if (presetJson["damperStorable"] != null) { damperStorable.val = presetJson["damperStorable"].AsFloat;}
                if (presetJson["maxVelocity"] != null) { maxVelocity.val = presetJson["maxVelocity"].AsFloat; }
                SetPhysicsValue();
            } catch (Exception e) {
                SuperController.LogMessage($"load preset error {e.Message}");
                }

        }
        private void ShowSaveBrowser() {


        }
        private void SavePreset(string path) {
            if (String.IsNullOrEmpty(path)) { return; }
            // _lastBrowseDir = path.Substring(0, path.LastIndexOfAny(new char[] { '/', '\\' })) + @"\";

            if ( !path.ToLower().EndsWith(saveExt.ToLower()))
            {
                path += "." + saveExt;
            }

            var presetJson = new JSONClass();
            // springStorable;damperStorable;maxVelocity;
            presetJson["springStorable"] = springStorable.val.ToString();
            presetJson["damperStorable"] = damperStorable.val.ToString();
            presetJson["maxVelocity"] = maxVelocity.val.ToString();

            this.SaveJSON(presetJson,path);
        }

        public void SetPhysicsValue() {
            var personList = GetSceneAtoms().FindAll(atom => atom.type == "Person");
            foreach (var atom in personList) {
                foreach (var actionName in atom.GetActionNames()) {
                    // SuperController.LogMessage("1 "+actionName);
                    // "springPercent" : "0.15","damperPercent" : "0.5","maxVeloctiy" : "0.4999999"
                    var allJointsControl = atom.GetStorableByID("AllJointsControl");
                    allJointsControl.SetFloatParamValue("springPercent",springStorable.val/100f);
                    allJointsControl.SetFloatParamValue("damperPercent",damperStorable.val/100f);
                    allJointsControl.SetFloatParamValue("maxVeloctiy",maxVelocity.val);
                    allJointsControl.CallAction("SetAllJointsPercentHoldSpring");
                    allJointsControl.CallAction("SetAllJointsPercentHoldDamper");
                    allJointsControl.CallAction("SetAllJointsMaxVelocity");
                }
            }
        }


        private void ActualInit()
        {
            // ...
            myNeedInit = false;
        }

        private void Update()
        {
            bool isLoading = SuperController.singleton.isLoading;
            bool isOrWasLoading = isLoading || myWasLoading;
            myWasLoading = isLoading;
            if (isOrWasLoading)
                myNeedInit = true;
            else if (myNeedInit)
                ActualInit();

            // ...
        }
    }
}
